'use strict';
import {filter} from '../../../src/utilities/filter';
import items from '../../data/items.json';

test('check initial data', () => {
    expect(items.length).toBe(50);
});

test.each([
    [{_id: 'part.11'}, null, 1],
    [{_id: 'part.0'}, null, 0],
    [{kind: 'product'}, null, 4],
    [{kind: 'module'}, null, 4],
    [{kind: 'part'}, null, 42],
    [{kind: 'fake'}, null, 0],
    [{type: 'Widerstand'}, null, 10],
    [{type: 'Widerstand', kind: 'product'}, null, 3],
    [{type: 'Diode'}, null, 5],
    [{type: 'Leiterplatte'}, null, 12],
    [{type: 'Transistor'}, null, 3],
    [{type: 'Kondensator'}, null, 5],
    [{type: 'Stiftleiste'}, null, 5],
    [{type: 'Schaltplan'}, null, 5],
    [{type: 'ICs'}, null, 5],
    [{type: 'ICs', kind: 'module'}, null, 1],
    [{type: 'Widerstand||Diode'}, null, 15],
    [{type: 'Leiterplatte||Transistor||Kondensator'}, null, 20],
    [{type: 'Stiftleiste||Schaltplan||ICs'}, null, 15],
    [{ref: {'_refId': 'mod.4'}}, null, 20],
    [{ref: {'_refId': 'mod.10'}}, null, 35],
    [{ref: {'_refId': 'mod.12'}}, null, 16],
    [{ref: {'_refId': 'prod.13'}}, null, 5],
    [{'ref._refId': 'mod.4'}, null, 20],
    [{'ref._refId': 'mod.4', kind: 'module'}, null, 2],
    [{'ref._refId': 'mod.10'}, null, 35],
    [{'ref._refId': 'mod.12'}, null, 16],
    [{'ref._refId': 'prod.13'}, null, 5],
    [{name: 'ab pane pulvis'}, {comparator: 'caseInsensitive'}, 1],
    [{name: 'Ab pane pulvis.'}, {comparator: 'caseInsensitive'}, 1],
    [{name: 'avaritiam nares'}, {comparator: 'caseInsensitive'}, 1],
    [{name: 'Avaritiam nares.'}, {comparator: 'caseInsensitive'}, 1],
    [{name: '!via'}, {comparator: 'caseInsensitive'}, 49],
    [{name: '!Via'}, {comparator: 'caseSensitive'}, 49],
    [{description: '!Via'}, {comparator: 'caseInsensitive'}, 48],
    [{description: '!via'}, {comparator: 'caseSensitive'}, 48],
    [{name: {$caseSensitive: 'ab pane pulvis'}}, null, 0],
    [{name: {$caseSensitive: 'Ab pane pulvis.'}}, null, 1],
    [{name: {$caseSensitive: 'avaritiam nares'}}, null, 0],
    [{name: {$caseSensitive: 'Avaritiam nares.'}}, null, 1],
    [{'ref': {amount: {$gte: 19}}}, null, 7],
    [{'ref': {amount: {$gt: 19}}}, null, 4],
    [{'ref': {amount: {$lte: 19}}}, null, 44],
    [{'ref': {amount: {$lt: 19}}}, null, 43],
    ['via', {comparator: 'caseInsensitive'}, 3],
    [{$: 'via'}, {comparator: 'caseInsensitive'}, 3],
    ['!via', {comparator: 'caseInsensitive'}, 47],
    [{$: '!via'}, {comparator: 'caseInsensitive'}, 47],
    ['!via', {comparator: 'caseSensitive'}, 48],
    [{$: '!via'}, {comparator: 'caseSensitive'}, 48],
    ['ab pane pulvis', {comparator: 'caseInsensitive'}, 1],
    ['Ab pane pulvis.', {comparator: 'caseInsensitive'}, 1],
    ['avaritiam nares', {comparator: 'caseInsensitive'}, 1],
    ['Avaritiam nares.', {comparator: 'caseInsensitive'}, 1],
    ['ab pane pulvis', {comparator: 'caseSensitive'}, 0],
    ['Ab pane pulvis.', {comparator: 'caseSensitive'}, 1],
    ['avaritiam nares', {comparator: 'caseSensitive'}, 0],
    ['Avaritiam nares.', {comparator: 'caseSensitive'}, 1],
    [{
        $or: {
            name: {'$caseInsensitive': '003090'},
            itemNumber: {'$caseInsensitive': '003090'},
            description: {'$caseInsensitive': '003090'}
        }
    }, null, 2],
    [{
        $or: {
            name: {'$caseInsensitive': '003090'},
            itemNumber: {'$caseInsensitive': '003090'},
            description: {'$caseInsensitive': '003090'}
        },
        revision: {
            _child: '!$exists'
        }
    }, null, 1],
    [{
        $and: {
            $or: {
                name: {'$caseInsensitive': '003090'},
                itemNumber: {'$caseInsensitive': '003090'},
                description: {'$caseInsensitive': '003090'}
            },
            revision: {
                _child: '!$exists'
            }
        }
    }, null, 1]
])(
    'filter with predicate %j and options %j = %i',
    (predicate, options, expected) => {
        expect(filter(items, predicate, options).length).toBe(expected);
    }
);
